@@ -24,9 +24,11 @@
 (require 'dash)
 (require 'ghub)
 (require 'graphql)
-(require 'subr-x)
 (require 'treepy)
 
+(eval-when-compile
+  (require 'subr-x))
+
 ;;; Api
 
 (cl-defun ghub-graphql (graphql &optional variables
@@ -78,7 +80,9 @@ behave as for `ghub-request' (which see)."
      createdAt
      updatedAt
      nameWithOwner
+     (parent nameWithOwner)
      description
+     homepageUrl
      (defaultBranchRef name)
      isArchived
      isFork
@@ -216,6 +220,7 @@ data as the only argument."
   (query     nil :read-only t)
   (variables nil :read-only t)
   (until     nil :read-only t)
+  (buffer    nil :read-only t)
   (pages     0   :read-only nil))
 
 (cl-defun ghub--graphql-vacuum (query variables callback
@@ -236,18 +241,23 @@ See Info node `(ghub)GraphQL Support'."
     :query     query
     :variables variables
     :until     until
-    :callback  (if narrow
+    :buffer    (current-buffer)
+    :callback  (let ((buf (current-buffer)))
+                 (if narrow
+                     (lambda (data)
+                       (let ((path narrow) key)
+                         (while (setq key (pop path))
+                           (setq data (cdr (assq key data)))))
+                       (ghub--graphql-set-mode-line buf nil)
+                       (funcall callback data))
                    (lambda (data)
-                     (let ((path narrow) key)
-                       (while (setq key (pop path))
-                         (setq data (cdr (assq key data)))))
-                     (funcall callback data))
-                 callback))))
+                     (ghub--graphql-set-mode-line buf nil)
+                     (funcall callback data)))))))
 
 (cl-defun ghub--graphql-retrieve (req &optional lineage cursor)
   (let ((p (cl-incf (ghub--graphql-req-pages req))))
     (when (> p 1)
-      (message "Fetching page %s..." p)))
+      (ghub--graphql-set-mode-line req "Fetching page %s" p)))
   (ghub--retrieve
    (let ((json-false nil))
      (ghub--encode-payload
@@ -281,7 +291,9 @@ See Info node `(ghub)GraphQL Support'."
                 (setq loc  (treepy-down loc))
                 (setq loc  (treepy-next loc)))
               (dolist (elt alist)
-                (cond ((keywordp (car elt)))
+                (cond ((eq (car elt) :alias)
+                       (push elt vars))
+                      ((keywordp (car elt)))
                       ((= (length elt) 3)
                        (push (list (nth 0 elt)
                                    (nth 1 elt)) vars)
@@ -415,26 +427,37 @@ See Info node `(ghub)GraphQL Support'."
 (defun ghub--graphql-encode (g)
   (if (symbolp g)
       (symbol-name g)
-    (let* ((object (car g))
+    (let* ((object (graphql--encode-object (car g)))
            (args   (and (vectorp (cadr g))
                         (cl-coerce (cadr g) 'list)))
-           (fields (if args (cddr g) (cdr g))))
-       (concat
-        (graphql--encode-object object)
-        (and args
-             (format " (\n%s)"
-                     (mapconcat (pcase-lambda (`(,key ,val))
-                                  (graphql--encode-argument key val))
-                                args ",\n")))
-        (and fields
-             (format " {\n%s\n}"
-                     (mapconcat #'ghub--graphql-encode fields "\n")))))))
+           (aliasp (cadr (assq :alias args)))
+           (fields (if args (cddr g) (cdr g)))
+           (fields (and fields
+                        (mapconcat #'ghub--graphql-encode fields "\n")))
+           (args   (and args
+                        (mapconcat (pcase-lambda (`(,key ,val))
+                                     (graphql--encode-argument key val))
+                                   args ",\n"))))
+      (if aliasp
+          (concat object ": " fields)
+        (concat object
+                (and args   (format " (\n%s)" args))
+                (and fields (format " {\n%s\n}" fields)))))))
 
 (defun ghub--alist-zip (root)
   (let ((branchp (lambda (elt) (and (listp elt) (listp (cdr elt)))))
         (make-node (lambda (_ children) children)))
     (treepy-zipper branchp #'identity make-node root)))
 
+(defun ghub--graphql-set-mode-line (buf string &rest args)
+  (when (ghub--graphql-req-p buf)
+    (setq buf (ghub--graphql-req-buffer buf)))
+  (when (buffer-live-p buf)
+    (with-current-buffer buf
+      (setq mode-line-process
+            (and string (concat " " (apply #'format string args))))
+      (force-mode-line-update t))))
+
 ;;; _
 (provide 'ghub-graphql)
 ;;; ghub-graphql.el ends here
